/*
 * grunt
 * http://gruntjs.com/
 *
 * Copyright (c) 2012 "Cowboy" Ben Alman
 * Licensed under the MIT license.
 * http://benalman.com/about/license/
 */

/*global phantom:true*/
/*jshint curly:false*/

(function(){
  'use strict';

  var fs = require('fs');

  // The temporary file used for communications.
  var tmpfile = phantom.args[0];

  // The page .html file to load.
  var url = phantom.args[1];

  // Extra, optionally overridable stuff.
  var options = JSON.parse(phantom.args[2] || {});

  // Default options.
  if (!options.timeout) { options.timeout = 5000; }

  // Keep track of the last time a client message was sent.
  var last = new Date();

  // Messages are sent to the parent by appending them to the tempfile.
  var sendMessage = function(arg) {
    var args = Array.isArray(arg) ? arg : [].slice.call(arguments);
    last = new Date();
    fs.write(tmpfile, JSON.stringify(args) + '\n', 'a');
  };

  // This allows grunt to abort if the PhantomJS version isn't adequate.
  sendMessage('private', 'version', phantom.version);

  // Abort if the page doesn't send any messages for a while.
  setInterval(function() {
    var since = new Date() - last;
    fs.write(tmpfile, '["tick", '+since+']'+ '\n', 'a');
    if (since > options.timeout) {
      sendMessage('fail.timeout');
      phantom.exit();
    }
  }, 100);

  // Create a new page.
  var page = require('webpage').create();

  // The client page must send its messages via alert(jsonstring).
  page.onAlert = function(args) {
    sendMessage(JSON.parse(args));
  };

  // Keep track if the client-side helper script already has been injected.
  var injected;
  page.onUrlChanged = function(newUrl) {
    injected = false;
    sendMessage('onUrlChanged', newUrl);
  };

  // Relay console logging messages.
  page.onConsoleMessage = function(message) {
    sendMessage('console', message);
  };

  // For debugging.
  page.onResourceRequested = function(request) {
    var method = request.method || 'GET';
    sendMessage('onResourceRequested', method + ' ' + request.url);
  };

  page.onResourceReceived = function(request) {
    var method = request.method || 'GET';
    if (request.stage === 'end') {
      sendMessage('onResourceReceived', method + ' ' + request.url);
    }
  };

  page.onError = function(msg, trace) {
    /*
     sendMessage(['error',msg]);
     var buffer = '';
     trace.forEach(function(line){
     buffer += ' > ' + line.file + ':' + line.line + (line.function ? 'in ' + line.function + '()' : '') + "\n";
     });
     sendMessage(['error',buffer]);
     */
    sendMessage('onError', msg, trace);
  };

  // Run when the page has finished loading.
  page.onLoadFinished = function(status) {
    if ( injected ) return;

    // The window has loaded.
    sendMessage('onLoadFinished', status);
    if (status === 'success') {
      if (options.inject && !injected) {
        // Inject client-side helper script, but only if it has not yet been
        // injected.
        options.inject.forEach(function(file){
          sendMessage('inject', file);
          page.injectJs(file);
        });
        injected = true;
      }
    } else {
      // File loading failure.
      sendMessage('fail.load', url);
      phantom.exit();
    }
  };
  page.open(url);
}());
